Struct LocalAddress

pub struct LocalAddress {
    address: AddrV2,
    last_connected: u64,
    state: AddressState,
    services: ServiceFlags,
    port: u16,
    pub id: usize,
}
How do we store peers locally

Fields

address: AddrV2

An actual address

last_connected: u64

Last time we successfully connected to this peer, only relevant is state == State::Tried

state: AddressState

Our local state for this peer, as defined in AddressState

services: ServiceFlags

Network services announced by this peer

port: u16

Network port this peers listens to

id: usize

Random id for this peer

Implementations

Source

impl LocalAddress

pub fn new( address: AddrV2, last_connected: u64, state: AddressState, services: ServiceFlags, port: u16, id: usize, ) -> LocalAddress

pub fn get_addrv2(&self) -> AddrV2

Get the [AddrV2] for this LocalAddress.

pub fn get_socket_address(&self) -> SocketAddr

Get the SocketAddr for this LocalAddress.

pub fn get_port(&self) -> u16

Get the port for this LocalAddress.

pub fn set_port(&mut self, port: u16)

Set the port for this LocalAddress.

pub fn get_services(&self) -> ServiceFlags

Get the [ServiceFlags] for this LocalAddress.

pub fn set_services(&mut self, services: ServiceFlags)

Set the [ServiceFlags] for this LocalAddress.

pub fn get_net_address(&self) -> IpAddr

Get the IpAddr for with this LocalAddress.

const fn is_routable(&self) -> bool

Return whether the address can be reached from our node

Some addresses are not reachable from the global internet, those includes documentation, reserved and private ones ranges. Since we can’t connect with them, there’s no point into keeping them

const fn is_routable_ipv4(ip: &Ipv4Addr) -> bool

Returns whether an ipv4 address is publicly routable

const fn is_routable_ipv6(ip: &Ipv6Addr) -> bool

Returns whether an ipv6 address is publicly routable

pub fn is_good_address(&self) -> bool

Return whether an address is good to connect to